@@ -5,6 +5,8 @@ from lightwood.mixer.neural_ts import NeuralTs
 from lightwood.mixer.lightgbm import LightGBM
 from lightwood.mixer.lightgbm_array import LightGBMArray
 from lightwood.mixer.sktime import SkTime
+from lightwood.mixer.arima import ARIMAMixer
+from lightwood.mixer.ets import ETSMixer
 from lightwood.mixer.nhits import NHitsMixer
 from lightwood.mixer.prophet import ProphetMixer
 from lightwood.mixer.regression import Regression
@@ -15,4 +17,4 @@ except Exception:
     QClassic = None
 
 __all__ = ['BaseMixer', 'Neural', 'NeuralTs', 'LightGBM', 'LightGBMArray', 'Unit', 'Regression',
-           'SkTime', 'QClassic', 'ProphetMixer', 'NHitsMixer']
+           'SkTime', 'QClassic', 'ProphetMixer', 'ETSMixer', 'ARIMAMixer', 'NHitsMixer']
